/**
 * LocationAPITools library, version 1.0.1
 * 
 * WWW: http://code.google.com/p/locationapitools
 *   
 * Summary:
 *   A collection of classes that simplify the development of location-aware mobile
 *   applications using the JSR-179 Location API on the J2ME (CLDC/MIDP) platform,
 *   e.g. on GPS-equipped smartphones.
 * 
 * Author:
 *   Matthias Stevens (Sony CSL Paris / Vrije Universiteit Brussel)
 *   Contact: matthias.stevens@gmail.com
 *  
 * Description:
 *   The primary use case is a location tracker that stores tracks in KML
 *   (cfr. Google Earth/Maps), CSV or plain text format.
 * 
 *   Package "com.sony.csl.j2me.locationapitools.demo" contains a MIDlet class that
 *   demonstrates how LocationAPITools can be used to create simple GPS tracker
 *   application that produces KML files. For examples of more advanced uses of this
 *   library refer to the NoiseTube project (see below, look for "NoiseTube MobileNoiseTagger").
 * 
 *   The code is known to be working on the following Symbian OS-based platforms:
 *     * S60 3rd Edition Feature Pack 1 (Symbian OS 9.2), e.g.: Nokia N95 and Nokia N95 8GB smartphones
 *     * S60 3rd Edition Feature Pack 2 (Symbian OS 9.3) 
 *   Although this is untested, the code should also work on other (non-Symbian/S60)
 *   J2ME CLDC/MIDP platforms/devices, as long as the JSR-179 Location API is supported
 *   on the platform/device in question.
 * 
 *   This library was developed within the scope of the NoiseTube project at
 *   Sony Computer Science Laboratory (CSL) Paris, for more information please refer to:
 *     * http://noisetube.net
 *     * http://code.google.com/p/noisetube
 *     * http://www.csl.sony.fr 
 *    
 * License: 
 *   Copyright 2008 Matthias Stevens (Sony CSL Paris / Vrije Universiteit Brussel)
 *   Licensed under the Apache License, Version 2.0 (the "License"); you may not use
 *   this file except in compliance with the License. You may obtain a copy of the
 *   License at http://www.apache.org/licenses/LICENSE-2.0.
 *   Unless required by applicable law or agreed to in writing, software distributed
 *   under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 *   CONDITIONS OF ANY KIND, either express or implied. See the License for the
 *   specific language governing permissions and limitations under the License.
 */

package com.sony.csl.j2me.locationapitools;

import java.util.Stack;

import javax.microedition.location.Coordinates;
import javax.microedition.location.Criteria;
import javax.microedition.location.Location;
import javax.microedition.location.LocationProvider;


/**
 * A class with useful methods that deal with specific status fields of classes in the JSR-179 Location API
 * 
 * @author mstevens (Sony CSL Paris / Vrije Universiteit Brussel)
 *
 */
public class LocationAPIUtilities
{
	
	/**
	 * Checks whether the JSR-179 Location API is supported.
	 * 
	 * @return a boolean indicating whether the JSR-179 Location API supported.
	 */
	public static boolean isLocationApiSupported()
	{
		String version = System.getProperty("microedition.location.version");
		return (version != null && !version.equals("")) ? true : false;
	}
	
	static public String locationProviderState(LocationProvider locationProvider)
	{
		return locationProviderState(locationProvider.getState());
	}
	
	static public String locationProviderState(int locationProviderState)
	{
		if(locationProviderState == LocationProvider.AVAILABLE)
			return "AVAILABLE";
		if(locationProviderState == LocationProvider.OUT_OF_SERVICE)
			return "OUT_OF_SERVICE";
		if(locationProviderState == LocationProvider.TEMPORARILY_UNAVAILABLE)
			return "TEMPORARILY_UNAVAILABLE";
		return "UNKNOWN_STATE";
	}
	
	static public String locationMethodFlags(Location location)
	{
		if(location.isValid())
		{
			return locationMethodFlags(location.getLocationMethod());
		}
		else
			return "Invalid location!";
	}
	
	static public String locationMethodFlags(int locationMethod)
	{
		Stack flags = new Stack();			
		//Query the bitfield to check all flags...
		if((locationMethod & Location.MTY_TERMINALBASED) !=0)
			flags.push("MTY_TERMINALBASED");			
		if((locationMethod & Location.MTY_NETWORKBASED) !=0)
			flags.push("MTY_NETWORKBASED");			
		if((locationMethod & Location.MTE_TIMEOFARRIVAL) !=0)
			flags.push("MTE_TIMEOFARRIVAL");			
		if((locationMethod & Location.MTE_TIMEDIFFERENCE) !=0)
			flags.push("MTE_TIMEDIFFERENCE");
		if((locationMethod & Location.MTE_SHORTRANGE) !=0)
			flags.push("MTE_SHORTRANGE");			
		if((locationMethod & Location.MTE_SATELLITE) !=0)
			flags.push("MTE_SATELLITE");			
		if((locationMethod & Location.MTE_CELLID) !=0)
			flags.push("MTE_CELLID");			
		if((locationMethod & Location.MTE_ANGLEOFARRIVAL) !=0)
			flags.push("MTE_ANGLEOFARRIVAL");			
		if((locationMethod & Location.MTA_UNASSISTED) !=0)
			flags.push("MTA_UNASSISTED");			
		if((locationMethod & Location.MTA_ASSISTED) !=0)
			flags.push("MTA_ASSISTED");
		//Write everything to a string...
		StringBuffer buffer = new StringBuffer(40);
		if(!flags.empty())
		{
			buffer.append((String) flags.pop());
			while(!flags.empty())
				buffer.append(", " + (String) flags.pop());
		}		
		return buffer.toString();
	}
	
	static public String getLocationAPISettings(LocationProxy locationProxy)
	{
		StringBuffer bff = new StringBuffer(500);
		//Criteria used to select LocationProvider:
		Criteria criteria = locationProxy.getProviderCriteria();
		bff.append("Horizontal Accuracy Criterium: " + ((criteria.getHorizontalAccuracy() == Criteria.NO_REQUIREMENT) ? "NO_REQUIREMENT" : criteria.getHorizontalAccuracy() + "m") + "; ");
		bff.append("Vertical Accuracy Criterium: " + ((criteria.getVerticalAccuracy() == Criteria.NO_REQUIREMENT) ? "NO_REQUIREMENT" : criteria.getVerticalAccuracy() + "m") + "; ");
		bff.append("Altitude Required Criterium: " + (criteria.isAltitudeRequired() ? "YES" : "NO") + "; ");
		bff.append("Speed and Course Required Criterium: " + (criteria.isSpeedAndCourseRequired() ? "YES" : "NO") + "; ");
		bff.append("AddressInfo Required Criterium: " + (criteria.isAddressInfoRequired() ? "YES" : "NO") + "; ");
		bff.append("Preferred Response Time Criterium: " + ((criteria.getPreferredResponseTime() == Criteria.NO_REQUIREMENT) ? "NO_REQUIREMENT" : criteria.getPreferredResponseTime() + "ms") + "; ");
		bff.append("Preferred Power Consumption Criterium: ");
		switch(criteria.getPreferredPowerConsumption())
		{
			case Criteria.NO_REQUIREMENT : bff.append("NO_REQUIREMENT"); break;
			case Criteria.POWER_USAGE_LOW : bff.append("POWER_USAGE_LOW"); break;
			case Criteria.POWER_USAGE_MEDIUM : bff.append("POWER_USAGE_MEDIUM"); break;
			case Criteria.POWER_USAGE_HIGH : bff.append("POWER_USAGE_HIGH"); break;
		}
		bff.append("; ");
		//Parameters for listening to LocationProvider:
		bff.append("Allowed To Cost Criterium: " + (criteria.isAllowedToCost() ? "YES" : "NO") + "; ");
		bff.append("Listening interval: " + locationProxy.getListeningInterval() + "s; ");
		bff.append("Listening timeout: " + locationProxy.getListeningTimeout() + "s; ");
		bff.append("Listening max-age: " + locationProxy.getListeningMaxAge() + "s");
		return bff.toString();
	}
	
	public static String coordinatesTuple(Coordinates coords, boolean forceAltitudeToZero, String separator)
	{
		return
			(new Double(coords.getLongitude())).toString() + separator +
			(new Double(coords.getLatitude())).toString() + 			
			((Float.isNaN(coords.getAltitude())) ?
				(forceAltitudeToZero ?
						(separator + "0") :
						"") :
				(separator + (new Float(coords.getAltitude())).toString()));
	}

}
