/**
 * LocationAPITools library, version 1.0.1
 * 
 * WWW: http://code.google.com/p/locationapitools
 *   
 * Summary:
 *   A collection of classes that simplify the development of location-aware mobile
 *   applications using the JSR-179 Location API on the J2ME (CLDC/MIDP) platform,
 *   e.g. on GPS-equipped smartphones.
 * 
 * Author:
 *   Matthias Stevens (Sony CSL Paris / Vrije Universiteit Brussel)
 *   Contact: matthias.stevens@gmail.com
 *  
 * Description:
 *   The primary use case is a location tracker that stores tracks in KML
 *   (cfr. Google Earth/Maps), CSV or plain text format.
 * 
 *   Package "com.sony.csl.j2me.locationapitools.demo" contains a MIDlet class that
 *   demonstrates how LocationAPITools can be used to create simple GPS tracker
 *   application that produces KML files. For examples of more advanced uses of this
 *   library refer to the NoiseTube project (see below, look for "NoiseTube MobileNoiseTagger").
 * 
 *   The code is known to be working on the following Symbian OS-based platforms:
 *     * S60 3rd Edition Feature Pack 1 (Symbian OS 9.2), e.g.: Nokia N95 and Nokia N95 8GB smartphones
 *     * S60 3rd Edition Feature Pack 2 (Symbian OS 9.3) 
 *   Although this is untested, the code should also work on other (non-Symbian/S60)
 *   J2ME CLDC/MIDP platforms/devices, as long as the JSR-179 Location API is supported
 *   on the platform/device in question.
 * 
 *   This library was developed within the scope of the NoiseTube project at
 *   Sony Computer Science Laboratory (CSL) Paris, for more information please refer to:
 *     * http://noisetube.net
 *     * http://code.google.com/p/noisetube
 *     * http://www.csl.sony.fr 
 *    
 * License: 
 *   Copyright 2008 Matthias Stevens (Sony CSL Paris / Vrije Universiteit Brussel)
 *   Licensed under the Apache License, Version 2.0 (the "License"); you may not use
 *   this file except in compliance with the License. You may obtain a copy of the
 *   License at http://www.apache.org/licenses/LICENSE-2.0.
 *   Unless required by applicable law or agreed to in writing, software distributed
 *   under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 *   CONDITIONS OF ANY KIND, either express or implied. See the License for the
 *   specific language governing permissions and limitations under the License.
 */

package com.sony.csl.j2me.locationapitools;

import java.util.Enumeration;
import java.util.Vector;

import javax.microedition.location.Location;
import javax.microedition.location.LocationProvider;

/**
 * @author mstevens
 *
 */
public class LocationCache implements LocationProxyListener
{
	
	private Vector cache;
	private long lastLocationTimeStamp;
	private int retentionTimeMilliSeconds;

	/**
	 * 
	 */
	public LocationCache(LocationProxy locationProxy, int retentionTimeSeconds) 
	{
		retentionTimeMilliSeconds = retentionTimeSeconds * 1000;
		cache = new Vector();
		locationProxy.addListener(this); //!!!
	}

	public void firstLocationFound(LocationProxy proxy, LocationProvider provider, Location location) { }

	public void locationProviderStateChanged(LocationProxy proxy, LocationProvider provider, int newState) { }

	public void locationUpdated(LocationProxy proxy, LocationProvider provider, Location location)
	{
		if(location != null && location.isValid()) //just to be sure
		{
			cache.addElement(new CacheEntry(location));
			lastLocationTimeStamp = location.getTimestamp();
			purgeOldLocations();
		}
	}
	
	private void purgeOldLocations()
	{
		if(!cache.isEmpty())
		{
			Vector cacheEntriesToRemove = new Vector();
			for(int e = 0; e < cache.size(); e++)
			{
				CacheEntry oldestCacheEntry = (CacheEntry) cache.elementAt(e);
				if(oldestCacheEntry.isOlderThan(lastLocationTimeStamp - retentionTimeMilliSeconds))
					cacheEntriesToRemove.addElement(oldestCacheEntry);
				else
					break;
			}
			Enumeration enumcacheEntriesToRemove = cacheEntriesToRemove.elements();
			while(enumcacheEntriesToRemove.hasMoreElements())
				cache.removeElement((CacheEntry) enumcacheEntriesToRemove.nextElement());
		}
	}

	public void proxyConnected(LocationProxy proxy, LocationProvider provider) { }

	public void proxyDisconnected(LocationProxy proxy) { }

	public boolean receivesInvalidLocations()
	{
		return false;
	}
	
	public Vector getCache()
	{
		return cache;
	}
	
	public boolean isEmpty()
	{
		return cache.isEmpty();
	}
	
	public int size()
	{
		return cache.size();
	}
	
	public Location getOldestLocation()
	{
		if(!cache.isEmpty())
			return ((CacheEntry) cache.elementAt(0)).getLocation();
		else
			return null;
	}
	
	public Location getNewestLocation()
	{
		if(!cache.isEmpty())
			return ((CacheEntry) cache.elementAt(cache.size() - 1)).getLocation();
		else
			return null;
	}
	
	public Location getLocationJustBefore(long timeStamp)
	{
		if(!cache.isEmpty())
		{
			if(timeStamp <= (lastLocationTimeStamp - retentionTimeMilliSeconds))
				return null;
			for(int e = cache.size() - 1; e >= 0; e--)
			{
				CacheEntry cacheEntry = (CacheEntry) cache.elementAt(e);
				if(cacheEntry.isOlderThan(timeStamp))
					return cacheEntry.getLocation();
			}
			return null;
		}
		else
			return null;		
	}
	
	public Location getLocationJustBeforeOrAt(long timeStamp)
	{
		if(!cache.isEmpty())
		{
			if(timeStamp < (lastLocationTimeStamp - retentionTimeMilliSeconds))
				return null;
			for(int e = cache.size() - 1; e >= 0; e--)
			{
				CacheEntry cacheEntry = (CacheEntry) cache.elementAt(e);
				if(cacheEntry.isOlderThanOrEquallyOldAs(timeStamp))
					return cacheEntry.getLocation();
			}
			return null;
		}
		else
			return null;	
	}
	
	public Location getLocationJustAfter(long timeStamp)
	{
		if(!cache.isEmpty())
		{
			if(timeStamp >= lastLocationTimeStamp)
				return null;
			for(int e = 0; e < cache.size(); e++)
			{
				CacheEntry cacheEntry = (CacheEntry) cache.elementAt(e);
				if(cacheEntry.isNewerThan(timeStamp))
					return cacheEntry.getLocation();
			}
			return null;
		}
		else
			return null;		
	}
	
	public Location getLocationJustAfterOrAt(long timeStamp)
	{
		if(!cache.isEmpty())
		{
			if(timeStamp > lastLocationTimeStamp)
				return null;
			for(int e = 0; e < cache.size(); e++)
			{
				CacheEntry cacheEntry = (CacheEntry) cache.elementAt(e);
				if(cacheEntry.isNewerThanOrEquallyOldAs(timeStamp))
					return cacheEntry.getLocation();
			}
			return null;
		}
		else
			return null;		
	}
	
	public Location getLocationClosestTo(long timeStamp)
	{
		if(!cache.isEmpty())
		{
			Location closestLocation = null;
			for(int e = 0; e < cache.size(); e++)
			{
				CacheEntry cacheEntry = (CacheEntry) cache.elementAt(e);
				if(closestLocation == null)
					closestLocation = cacheEntry.getLocation();
				else
				{
					if(Math.abs(timeStamp - cacheEntry.getTimeStamp()) < Math.abs(timeStamp - closestLocation.getTimestamp()))
						closestLocation = cacheEntry.getLocation();
				}
			}
			return closestLocation;
		}
		else
			return null;
	}
	
	public Location[] getLocationsBetween(long begin, long end)
	{
		return getLocationsBetween(begin, end, true);
	}
	
	public Location[] getLocationsBetween(long begin, long end, boolean inclusive)
	{
		if(!cache.isEmpty())
		{
			Vector locationsInBetween = new Vector();
			for(int e = 0; e < cache.size(); e++)
			{
				CacheEntry cacheEntry = (CacheEntry) cache.elementAt(e);
				if(inclusive)
				{
					if(cacheEntry.isNewerThanOrEquallyOldAs(begin) && cacheEntry.isOlderThanOrEquallyOldAs(end))
						locationsInBetween.addElement(cacheEntry.getLocation());
				}
				else
				{
					if(cacheEntry.isNewerThan(begin) && cacheEntry.isOlderThan(end))
						locationsInBetween.addElement(cacheEntry.getLocation());
				}
			}
			if(!locationsInBetween.isEmpty())
			{
				Location[] locationsInBetween_return = new Location[locationsInBetween.size()];
				locationsInBetween.copyInto(locationsInBetween_return);
				return locationsInBetween_return;
			}
			else
				return null;
		}
		else
			return null;	
	}
	
	private class CacheEntry
	{
	
		private long timeStamp;
		private Location location;
		
		public CacheEntry(Location location)
		{
			this.location = location;
			this.timeStamp = location.getTimestamp();
		}
		
		public long getTimeStamp()
		{
			return timeStamp;
		}
		
		public Location getLocation()
		{
			return location;
		}
		
		public boolean isOlderThan(long timeStamp)
		{
			return (this.timeStamp < timeStamp);
		}
		
		public boolean isOlderThanOrEquallyOldAs(long timeStamp)
		{
			return (this.timeStamp <= timeStamp);
		}
		
		public boolean isNewerThan(long timeStamp)
		{
			return (this.timeStamp > timeStamp);
		}
		
		public boolean isNewerThanOrEquallyOldAs(long timeStamp)
		{
			return (this.timeStamp >= timeStamp);
		}
		
	}

}
