/**
 * LocationAPITools library, version 1.0.1
 * 
 * WWW: http://code.google.com/p/locationapitools
 *   
 * Summary:
 *   A collection of classes that simplify the development of location-aware mobile
 *   applications using the JSR-179 Location API on the J2ME (CLDC/MIDP) platform,
 *   e.g. on GPS-equipped smartphones.
 * 
 * Author:
 *   Matthias Stevens (Sony CSL Paris / Vrije Universiteit Brussel)
 *   Contact: matthias.stevens@gmail.com
 *  
 * Description:
 *   The primary use case is a location tracker that stores tracks in KML
 *   (cfr. Google Earth/Maps), CSV or plain text format.
 * 
 *   Package "com.sony.csl.j2me.locationapitools.demo" contains a MIDlet class that
 *   demonstrates how LocationAPITools can be used to create simple GPS tracker
 *   application that produces KML files. For examples of more advanced uses of this
 *   library refer to the NoiseTube project (see below, look for "NoiseTube MobileNoiseTagger").
 * 
 *   The code is known to be working on the following Symbian OS-based platforms:
 *     * S60 3rd Edition Feature Pack 1 (Symbian OS 9.2), e.g.: Nokia N95 and Nokia N95 8GB smartphones
 *     * S60 3rd Edition Feature Pack 2 (Symbian OS 9.3) 
 *   Although this is untested, the code should also work on other (non-Symbian/S60)
 *   J2ME CLDC/MIDP platforms/devices, as long as the JSR-179 Location API is supported
 *   on the platform/device in question.
 * 
 *   This library was developed within the scope of the NoiseTube project at
 *   Sony Computer Science Laboratory (CSL) Paris, for more information please refer to:
 *     * http://noisetube.net
 *     * http://code.google.com/p/noisetube
 *     * http://www.csl.sony.fr 
 *    
 * License: 
 *   Copyright 2008 Matthias Stevens (Sony CSL Paris / Vrije Universiteit Brussel)
 *   Licensed under the Apache License, Version 2.0 (the "License"); you may not use
 *   this file except in compliance with the License. You may obtain a copy of the
 *   License at http://www.apache.org/licenses/LICENSE-2.0.
 *   Unless required by applicable law or agreed to in writing, software distributed
 *   under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 *   CONDITIONS OF ANY KIND, either express or implied. See the License for the
 *   specific language governing permissions and limitations under the License.
 */

package com.sony.csl.j2me.locationapitools.demo;

import javax.microedition.lcdui.*;
import javax.microedition.location.Coordinates;
import javax.microedition.location.Location;
import javax.microedition.location.LocationProvider;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import com.sony.csl.j2me.locationapitools.*;
import com.sony.csl.j2me.locationapitools.tracking.*;
import com.sony.csl.j2me.utils.StringUtils;

/**
 * A simple application (MIDlet) to demonstrate the functionality of the LocationAPI Tools library 
 * 
 * @author mstevens
 *
 */
public class DemoMIDlet extends MIDlet implements CommandListener, LocationProxyListener
{
		
	//A static reference to Display object
    private static Display display = null;
    
	//Config:
	public static final String DEFAULT_PATH = "E:/LocationAPITools Demo/";
	public static final String DEFAULT_TRACKNAME = "DemoTrack";
    
    //GUI:
    private Command cmdLocationInfo;
    private Command cmdNewTrack;
    private Command cmdStopTracking;
    private Command cmdAbout;
    private Command cmdExit;
    private Form frmMain;
    private NewTrackForm frmNewTrack;
    private LocationInfoForm frmLocationInfo;
    private Ticker ticker;
    private StringItem stritLog;

    
    //Location stuff:
    private LocationProxy locationProxy;  
	TrackWriter trackWriter;
	
	public DemoMIDlet()
	{
		super();
		display = Display.getDisplay(this);
		
		//GUI:
		frmMain = new Form("LocationAPITools Demo");
		stritLog = new StringItem("Status log:", "");
		stritLog.setLayout(Item.LAYOUT_EXPAND | Item.LAYOUT_VEXPAND);
		stritLog.setFont(Font.getFont(Font.FACE_PROPORTIONAL, Font.STYLE_PLAIN, Font.SIZE_SMALL));
		cmdLocationInfo = new Command("Location info...", Command.ITEM, 1);
		cmdNewTrack = new Command("Start Tracking", Command.ITEM, 1);
		cmdStopTracking = new Command("Stop Tracking", Command.ITEM, 1);
		cmdAbout = new Command("About...", Command.ITEM, 1);
		cmdExit = new Command("Exit", Command.EXIT, 1);
		ticker = new Ticker("Welcome to LocationAPITools DemoMIDlet");
        frmMain.setTicker(ticker);
        frmMain.append(stritLog);
        frmMain.setCommandListener(this);
		frmMain.addCommand(cmdExit);
		frmMain.addCommand(cmdAbout);
		
		//Other forms:
		frmNewTrack = new NewTrackForm(this);
		frmLocationInfo = new LocationInfoForm(this);
	}
	
	/**
	 * Getter method for Display reference.
	 * 
	 * @return reference to Display object.
	 */
	public static Display getDisplay()
	{
		return display;
	}

	protected void startApp() throws MIDletStateChangeException
    {
        display = Display.getDisplay(this);		
		display.setCurrent(frmMain);
        
		printLogLn("Welcome to LocationAPITools DemoMIDlet");
		
		//Location API...
		if(LocationAPIUtilities.isLocationApiSupported())
		{
			printLogLn("Location API is supported");
			try
			{
				frmLocationInfo = new LocationInfoForm(this);
				locationProxy = LocationProxy.getInstance();
				locationProxy.addListener(this);
				if(locationProxy.connect())
				{
					printLogLn("Connected to Location provider");
					frmMain.addCommand(cmdLocationInfo);
					frmMain.addCommand(cmdNewTrack); //enable tracking...
				}
				else
				{
					printLogLn("Could not connect to Location provider");
				}
			}
			catch(Exception e)
			{
				printLogLn("Exception: " + e.getMessage());
			}
		}
		else
		{
			printLogLn("Location API is not supported");
		}
	}
	
	protected void pauseApp()
	{
	}

	protected void destroyApp(boolean unconditional)
	{
		stopTracking();
		notifyDestroyed();
	}

	public void startTracking()
	{
		try
		{
			if(trackWriter != null && trackWriter.isTracking())
				stopTracking();
			switch (frmNewTrack.getTrackFormat())
			{
				case (NewTrackForm.KML_FORMAT) : trackWriter = new KMLTrackWriter(locationProxy, frmNewTrack.getPath(), frmNewTrack.getTrackName(), false, new DefaultKMLStyle()); break;
				case (NewTrackForm.CSV_FORMAT) : trackWriter = new CSVTrackWriter(locationProxy, frmNewTrack.getPath(), frmNewTrack.getTrackName(), false); break;
				case (NewTrackForm.TXT_FORMAT) : trackWriter = new TXTTrackWriter(locationProxy, frmNewTrack.getPath(), frmNewTrack.getTrackName(), false); break;
				default : throw new Exception("Unknown track format");
			}			
			trackWriter.startTracking();			
			if(trackWriter.isTracking())
			{	
				frmMain.removeCommand(cmdNewTrack);
				frmMain.addCommand(cmdStopTracking);
				printLogLn("Tracking started");
			}
		}
		catch(Exception e)
		{
			printLogLn("Exception: " + e.getMessage());
		}
	}
	
	public void stopTracking()
	{
		if(trackWriter != null && trackWriter.isTracking())
		{
			trackWriter.stopTracking();
			trackWriter = null;
			frmMain.removeCommand(cmdStopTracking);
			frmMain.addCommand(cmdNewTrack);
			printLogLn("Tracking stopped");
		}		
	}
	
	public void commandAction(Command cmd, Displayable disp)
	{
		if(cmd == cmdLocationInfo)
		{
			display.setCurrent(frmLocationInfo);
		}
		if(cmd == cmdNewTrack)
		{
			display.setCurrent(frmNewTrack);
		}
		if(cmd == cmdStopTracking)
		{
			stopTracking();
		}
		if(cmd == cmdAbout)
		{
			//"About" messages:
			printLogLn("");
			printLogLn("LocationAPITools DemoMIDlet");
			printLogLn(" Using LocationAPITools  v1.0.0");
			printLogLn(" Copyright 2008 Matthias Stevens");
			printLogLn(" More info on:\n  http://code.google.com/p/locationapitools");
			printLogLn("");
		}
		if (cmd == cmdExit)
		{
			destroyApp(false);
		}
	}
	
    public Form getMainForm()
    {
        return frmMain;
    }
	
	public void printLogLn(String line)
	{
		stritLog.setText(stritLog.getText() + line + "\n");
	}

	public void firstLocationFound(LocationProxy proxy, LocationProvider provider, Location location)
	{
		printLogLn("First location found");
	}

	public void locationProviderStateChanged(LocationProxy proxy, LocationProvider provider, int newState)
	{	
		//...
	}

	public void locationUpdated(LocationProxy proxy, LocationProvider provider,	Location location)
	{
		frmLocationInfo.setStrNumberOfLocations(String.valueOf(proxy.getNumberOfValidLocations()));
		Coordinates qualCoords = location.getQualifiedCoordinates();
		frmLocationInfo.setStrCurrentLocation(
			"Longitude: " + qualCoords.getLongitude() + "\n"
			+ "Latitude: " + qualCoords.getLatitude() + "\n"
			+ (!Float.isNaN(qualCoords.getAltitude()) ? "Altitude: " + qualCoords.getAltitude() + "m" : "") + "\n"
			+ "(captured at: " + StringUtils.formatTime(location.getTimestamp(), ":") + ")");
	}

	public void proxyConnected(LocationProxy proxy, LocationProvider provider)
	{
		//...
	}

	public void proxyDisconnected(LocationProxy proxy)
	{
		//...
	}

	public boolean receivesInvalidLocations()
	{
		return false;
	}
    
}