/**
 * LocationAPITools library, version 1.0.1
 * 
 * WWW: http://code.google.com/p/locationapitools
 *   
 * Summary:
 *   A collection of classes that simplify the development of location-aware mobile
 *   applications using the JSR-179 Location API on the J2ME (CLDC/MIDP) platform,
 *   e.g. on GPS-equipped smartphones.
 * 
 * Author:
 *   Matthias Stevens (Sony CSL Paris / Vrije Universiteit Brussel)
 *   Contact: matthias.stevens@gmail.com
 *  
 * Description:
 *   The primary use case is a location tracker that stores tracks in KML
 *   (cfr. Google Earth/Maps), CSV or plain text format.
 * 
 *   Package "com.sony.csl.j2me.locationapitools.demo" contains a MIDlet class that
 *   demonstrates how LocationAPITools can be used to create simple GPS tracker
 *   application that produces KML files. For examples of more advanced uses of this
 *   library refer to the NoiseTube project (see below, look for "NoiseTube MobileNoiseTagger").
 * 
 *   The code is known to be working on the following Symbian OS-based platforms:
 *     * S60 3rd Edition Feature Pack 1 (Symbian OS 9.2), e.g.: Nokia N95 and Nokia N95 8GB smartphones
 *     * S60 3rd Edition Feature Pack 2 (Symbian OS 9.3) 
 *   Although this is untested, the code should also work on other (non-Symbian/S60)
 *   J2ME CLDC/MIDP platforms/devices, as long as the JSR-179 Location API is supported
 *   on the platform/device in question.
 * 
 *   This library was developed within the scope of the NoiseTube project at
 *   Sony Computer Science Laboratory (CSL) Paris, for more information please refer to:
 *     * http://noisetube.net
 *     * http://code.google.com/p/noisetube
 *     * http://www.csl.sony.fr 
 *    
 * License: 
 *   Copyright 2008 Matthias Stevens (Sony CSL Paris / Vrije Universiteit Brussel)
 *   Licensed under the Apache License, Version 2.0 (the "License"); you may not use
 *   this file except in compliance with the License. You may obtain a copy of the
 *   License at http://www.apache.org/licenses/LICENSE-2.0.
 *   Unless required by applicable law or agreed to in writing, software distributed
 *   under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 *   CONDITIONS OF ANY KIND, either express or implied. See the License for the
 *   specific language governing permissions and limitations under the License.
 */

package com.sony.csl.j2me.locationapitools.tracking;

import java.util.Enumeration;
import java.util.Vector;

import javax.microedition.location.*;

import com.sony.csl.j2me.locationapitools.*;
import com.sony.csl.j2me.utils.*;
import com.sony.csl.j2me.utils.io.XMLUtils;
import com.sony.csl.j2me.utils.io.HTMLUtils;

/**
 * A TrackWriter subclass that stores a location track as a KML file for use with Google Earth or Google Maps
 * 
 * @author mstevens (Sony CSL Paris / Vrije Universiteit Brussel)
 *
 */
public class KMLTrackWriter extends TrackWriter
{
	
	static public final String DEFAULT_CHARACTER_ENCODING = "UTF-8";
	
	protected KMLStyle kmlStyle;
	private Vector coordinateStrings;

	public KMLTrackWriter(LocationProxy locationProxy, String trackFileFolderPath, String trackName) throws Exception
	{
		this(locationProxy, trackFileFolderPath, trackName, false);
	}
	
	public KMLTrackWriter(LocationProxy locationProxy, String trackFileFolderPath, String trackName, boolean tracksInvalids) throws Exception
	{
		this(locationProxy, trackFileFolderPath, trackName, tracksInvalids, new DefaultKMLStyle(), DEFAULT_CHARACTER_ENCODING);
	}
	
	public KMLTrackWriter(LocationProxy locationProxy, String trackFileFolderPath, String trackName, boolean tracksInvalids, KMLStyle kmlStyle) throws Exception
	{
		this(locationProxy, trackFileFolderPath, trackName, tracksInvalids, kmlStyle, DEFAULT_CHARACTER_ENCODING);
	}
	
	public KMLTrackWriter(LocationProxy locationProxy, String trackFileFolderPath, String trackName, boolean tracksInvalids, KMLStyle kmlStyle, String characterEncoding) throws Exception
	{
		super(locationProxy, trackFileFolderPath, trackName, tracksInvalids);
		this.kmlStyle = kmlStyle;
		this.characterEncoding = characterEncoding; 
	}
	
	/**
	 * @return the kmlStyle
	 */
	public KMLStyle getKmlStyle()
	{
		return kmlStyle;
	}

	/**
	 * @param kmlStyle the kmlStyle to set
	 */
	public void setKmlStyle(KMLStyle kmlStyle) throws Exception
	{
		if(!isTracking())
			this.kmlStyle = kmlStyle;
		else
			throw new Exception("Cannot change KML style while tracking");
	}

	/**
	 * @see com.sony.csl.j2me.locationapitools.tracking.TrackWriter#initializeBeforeStart()
	 */
	protected void initializeBeforeStart()
	{
		coordinateStrings = new Vector();
	}
	
	/** 
	 * @see com.sony.csl.j2me.locationapitools.tracking.TrackWriter#getFileExtension()
	 */
	protected String getFileExtension()
	{
		return "kml";
	}
	
	/** 
	 * @see com.sony.csl.j2me.locationapitools.tracking.TrackWriter#writeHeader()
	 */
	protected void writeHeader()
	{
		CustomStringBuffer bff = new CustomStringBuffer(2700);
		//XML & KML file headers
		bff.appendLine("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		bff.appendLine("<kml xmlns=\"http://www.opengis.net/kml/2.2\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">");
		bff.appendTabbedLine("<Document id=\"doc_mnt-track\">", 1);
		bff.appendTabbedLine("<name>" + XMLUtils.escapeCharacters(trackName) + "</name>", 2);
		bff.appendTabbedLine("<visibility>1</visibility>", 2);
		bff.appendTabbedLine("<open>1</open>", 2);
		String platform = System.getProperty("microedition.platform");
		bff.appendTabbedLine(	"<Snippet>Created with " +
								XMLUtils.escapeCharacters(kmlStyle.getApplicationName()) +
								" on " + XMLUtils.timeDateValue(trackStartTime) +
								((platform != null && !platform.equals("")) ?
										" (using device: " + XMLUtils.escapeCharacters(platform) + ")" :
										"") +
								"</Snippet>", 2);
		bff.appendTabbedLine("<description>", 2);
		bff.appendTabbedLine("<![CDATA[", 3);
		bff.appendTabbedLine(	"KML track created with " +
								XMLUtils.escapeCharacters(kmlStyle.getApplicationName()) +
								" on " + XMLUtils.timeDateValue(trackStartTime) +
								((platform != null && !platform.equals("")) ?
										" (using device: " + XMLUtils.escapeCharacters(platform) + ")" :
										""), 4);
		bff.appendTabbedLine("]]>", 3);
		bff.appendTabbedLine("</description>", 2);
		//Style:
		bff.append(kmlStyle.getStyleDefinitions(2));
		//Open the folder for the location placemarks:
		bff.appendTabbedLine("<Folder id=\"f_locations\">", 2);
		bff.appendTabbedLine("<name>Locations</name>", 3);
		bff.appendTabbedLine("<open>0</open>", 3);
		//Write to file
		writeToFile(bff.toString());
	}
	
	/** 
	 * @see com.sony.csl.j2me.locationapitools.tracking.TrackWriter#writeLocation(javax.microedition.location.Location)
	 */
	protected void writeLocation(Location location)
	{
		CustomStringBuffer bff = new CustomStringBuffer(1800);
		if(location.isValid())
		{
			QualifiedCoordinates qualCoords = location.getQualifiedCoordinates();
			String coordinateTuple = coordinatesTuple(qualCoords);
			//store a string with the coordinates for the track that will be written in the footer
			coordinateStrings.addElement(coordinateTuple);
			//write the placemark tag
			bff.appendTabbedLine("<Placemark id=\"pm_loc" + numberOfLocations + "\">", 3);
			bff.appendTabbedLine("<name>Location " + numberOfLocations + "</name>", 4);
			bff.appendTabbedLine("<Snippet></Snippet>", 4); //avoids that part of the description is shown in the places list (to keep things tidy)
			bff.appendTabbedLine("<description>", 4);
			bff.appendTabbedLine("<![CDATA[", 5);
			bff.appendTabbedLine("<table>", 6);
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("Time: " + XMLUtils.timeDateValue(location.getTimestamp())), 7);
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("Longitude: " + qualCoords.getLongitude() + ""), 7);
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("Latitude: " + qualCoords.getLatitude() + ""), 7);
			if(!Float.isNaN(qualCoords.getAltitude()))
				bff.appendTabbedLine(HTMLUtils.htmlTableRow("Altitude: " + qualCoords.getAltitude() + "m"), 7);
			if(!Float.isNaN(location.getCourse()))
				bff.appendTabbedLine(HTMLUtils.htmlTableRow("Heading/Course: " + location.getCourse() + ""), 7);
			if(!Float.isNaN(qualCoords.getHorizontalAccuracy()))
				bff.appendTabbedLine(HTMLUtils.htmlTableRow("Horizontal Accuracy: " + qualCoords.getHorizontalAccuracy() + "m"), 7);
			if(!Float.isNaN(qualCoords.getVerticalAccuracy()))
				bff.appendTabbedLine(HTMLUtils.htmlTableRow("Vertical Accuracy: " + qualCoords.getVerticalAccuracy() + "m"), 7);
			if(!Float.isNaN(location.getSpeed()))
				bff.appendTabbedLine(HTMLUtils.htmlTableRow("Speed (captured by sensor): " + location.getSpeed() + "m/s (" + (location.getSpeed() * 3.6) + "km/h)"), 7);
			if(lastValidLocation != null)
			{
				float metersFromLastLocation = lastValidLocation.getQualifiedCoordinates().distance(location.getQualifiedCoordinates());
				long secondsSinceFromLastLocation = (location.getTimestamp() - lastValidLocation.getTimestamp()) / 1000;
				float metersPerSecond = metersFromLastLocation / secondsSinceFromLastLocation;
				bff.appendTabbedLine(HTMLUtils.htmlTableRow("Travel speed from previous location: " + metersPerSecond + "m/s (" + (metersPerSecond * 3.6) + "km/h)"), 7);
			}
			if(location.getAddressInfo() != null)
				bff.appendTabbedLine(HTMLUtils.htmlTableRow("AddressInfo: " + XMLUtils.escapeCharacters(location.getAddressInfo().toString())), 7);
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("LocationMethod flags: " + LocationAPIUtilities.locationMethodFlags(location)), 7);
			bff.appendTabbedLine("</table>", 6);
			bff.appendTabbedLine("]]>", 5);
			bff.appendTabbedLine("</description>", 4);
			bff.appendTabbedLine("<TimeStamp>", 4);
			bff.appendTabbedLine("<when>" + XMLUtils.timeDateValue(location.getTimestamp()) + "</when>", 5);
			bff.appendTabbedLine("</TimeStamp>", 4);
			bff.appendTabbedLine("<styleUrl>" + kmlStyle.getLocationStyleURL() + "</styleUrl>", 4);
			bff.appendTabbedLine("<Point>", 4);
			bff.appendTabbedLine("<altitudeMode>clampToGround</altitudeMode>", 5);
			bff.appendTabbedLine("<coordinates>" + coordinateTuple + "</coordinates>", 5);
			bff.appendTabbedLine("</Point>", 4);			
			bff.appendTabbedLine("</Placemark>", 3);
		}
		else
			bff.appendLine(XMLUtils.comment("Location " + numberOfLocations + "(received at " + XMLUtils.timeDateValue(System.currentTimeMillis()) + ") is invalid!", 3));
		//Write to file
		writeToFile(bff.toString());
	}
	
	/**
	 * @see com.sony.csl.j2me.locationapitools.tracking.TrackWriter#writePaused(java.lang.String)
	 */
	protected void writePaused(String reason)
	{
		writeToFile(XMLUtils.comment("Tracking paused at " + XMLUtils.timeDateValue(System.currentTimeMillis())+ ((reason != null) ? (" reason: " + XMLUtils.escapeCharacters(reason)) : ""), 3) + "\n");		
	}
	
	/**
	 * @see com.sony.csl.j2me.locationapitools.tracking.TrackWriter#writeResumed()
	 */
	protected void writeResumed()
	{
		writeToFile(XMLUtils.comment("Tracking resumed at " + XMLUtils.timeDateValue(System.currentTimeMillis()), 3) + "\n");		
	}

	/**
	 * @see com.sony.csl.j2me.locationapitools.tracking.TrackWriter#writeProviderStateChanged(javax.microedition.location.LocationProvider, int)
	 */
	protected void writeProviderStateChanged(LocationProvider provider, int newState)
	{
		writeToFile(XMLUtils.comment("LocationProvider state changed to " + LocationAPIUtilities.locationProviderState(newState) + " at " + XMLUtils.timeDateValue(System.currentTimeMillis()), 3) + "\n");		
	}

	/** 
	 * @see com.sony.csl.j2me.locationapitools.tracking.TrackWriter#writeFooter()
	 */
	protected void writeFooter()
	{
		CustomStringBuffer bff = new CustomStringBuffer(5000 + (60 * numberOfLocations));
		//Close the placemarks folder:
		bff.appendTabbedLine("</Folder>", 2);
		//Add summarizing graphics: 
		if(numberOfValidLocations >= 2) //minimum 2 locations needed for a meaningful summary
		{
			//Draw the track as a graphical line:
			bff.append(getTrackLinePlacemark());
			//Draw the track bounding box polygon:
			bff.append(getBoundingBoxPlacemark());
		}
		//Add global summary (description) for the Document:
		bff.append(getTrackDescription());
		//Add TimeSpan (not sure if it makes sense for GoogleEarth to put that here, but the schema allows it and maybe it will be useful for our own tools:
		if(firstValidLocation != null && lastValidLocation != null)
			bff.append(kmlTimeSpan(firstValidLocation.getTimestamp(), lastValidLocation.getTimestamp(), 2));
		//Add additional contents to be provided by subclasses:
		bff.append(getAdditionalTrackContents(2));
		//Close the document:
		bff.appendTabbedLine("</Document>", 1);
		bff.appendLine("</kml>");
		//Write to file
		writeToFile(bff.toString());
	}
	
	protected String getAdditionalTrackContents(int tabs) //to be overridden by subclasses
	{
		return "";
	}
	
	private String getTrackLinePlacemark()
	{
		CustomStringBuffer bff = new CustomStringBuffer(1000 + (60 * numberOfLocations));
		bff.appendTabbedLine("<Placemark id=\"pm_trackline\">", 2);
		bff.appendTabbedLine("<name>Track line</name>", 3);
		bff.appendTabbedLine("<visibility>1</visibility>", 3);
		bff.appendTabbedLine("<Snippet>A line interconnecting all" + (tracksInvalids ? " valid" : "") + " locations on this track</Snippet>", 3);
		bff.appendTabbedLine("<description>", 3);
		bff.appendTabbedLine("<![CDATA[", 4);
		bff.appendTabbedLine("A line interconnecting all " + numberOfLocations + " locations on this track.", 5);
		bff.appendTabbedLine("]]>", 4);
		bff.appendTabbedLine("</description>", 3);
		bff.append(kmlTimeSpan(firstLocation.getTimestamp(), lastLocation.getTimestamp(), 3));
		bff.appendTabbedLine("<styleUrl>" + kmlStyle.getTrackLineStyleURL() + "</styleUrl>", 3);
		bff.appendTabbedLine("<LineString>", 3);
		bff.appendTabbedLine("<altitudeMode>clampToGround</altitudeMode>", 4);
		bff.appendTabbedLine("<tessellate>1</tessellate>", 4);
		bff.appendTabbed("<coordinates>", 4);
		Enumeration coordinateStringsEnum = coordinateStrings.elements();
		while(coordinateStringsEnum.hasMoreElements())
			bff.append(((String) coordinateStringsEnum.nextElement()) + " ");
		bff.appendLine("</coordinates>");
		bff.appendTabbedLine("</LineString>", 3);
		bff.appendTabbedLine("</Placemark>", 2);
		return bff.toString();
	}
	
	private String getBoundingBoxPlacemark()
	{
		CustomStringBuffer bff = new CustomStringBuffer(2400);
		bff.appendTabbedLine("<Placemark id=\"pm_boundingbox\">", 2);
		bff.appendTabbedLine("<name>Bounding box polygon</name>", 3);
		bff.appendTabbedLine("<visibility>0</visibility>", 3);
		bff.appendTabbedLine("<Snippet>A polygon that marks the bounding box of the track</Snippet>", 3);
		bff.appendTabbedLine("<description>", 3);
		bff.appendTabbedLine("<![CDATA[", 4);
		bff.appendTabbedLine("A polygon that marks the bounding box of the track. The box is defined by:</br>", 5);
		bff.appendTabbedLine("<table>", 5);
		bff.appendTabbedLine(HTMLUtils.htmlTableRow("\"Left-Bottom corner\" (minLongitude,minLatitude): " + minLongitude + "," + minLatitude), 6);
		bff.appendTabbedLine(HTMLUtils.htmlTableRow("\"Right-Bottom corner\" (maxLongitude,minLatitude): " + maxLongitude + "," + minLatitude), 6);
		bff.appendTabbedLine(HTMLUtils.htmlTableRow("\"Right-Top corner\" (maxLongitude,maxLatitude): " + maxLongitude + "," + maxLatitude), 6);
		bff.appendTabbedLine(HTMLUtils.htmlTableRow("\"Left-Top corner\" (minLongitude,maxLatitude): " + minLongitude + "," + maxLatitude), 6);		
		bff.appendTabbedLine("</table>", 5);
		bff.appendTabbedLine("]]>", 4);
		bff.appendTabbedLine("</description>", 3);
		bff.append(kmlTimeSpan(firstLocation.getTimestamp(), lastLocation.getTimestamp(), 3));
		bff.appendTabbedLine("<styleUrl>" + kmlStyle.getBoundingBoxStyleURL() + "</styleUrl>", 3);
		bff.appendTabbedLine("<Polygon>", 3);
		bff.appendTabbedLine("<altitudeMode>clampToGround</altitudeMode>", 4);
		bff.appendTabbedLine("<tessellate>1</tessellate>", 4);
		bff.appendTabbedLine("<outerBoundaryIs>", 4);
		bff.appendTabbedLine("<LinearRing>", 5);
		bff.appendTabbedLine("<coordinates>", 6);
		bff.appendTabbedLine(minLongitude + "," + minLatitude + " ", 7); //"Left-Bottom"		
		bff.appendTabbedLine(maxLongitude + "," + minLatitude + " ", 7); //"Right-Bottom"
		bff.appendTabbedLine(maxLongitude + "," + maxLatitude + " ", 7); //"Right-Top"
		bff.appendTabbedLine(minLongitude + "," + maxLatitude + " ", 7); //"Left-Top"
		bff.appendTabbedLine(minLongitude + "," + minLatitude + " ", 7); //"Left-Bottom" (again, to close the linering)
		bff.appendTabbedLine("</coordinates>", 6);
		bff.appendTabbedLine("</LinearRing>", 5);
		bff.appendTabbedLine("</outerBoundaryIs>", 4);
		bff.appendTabbedLine("</Polygon>", 3);
		bff.appendTabbedLine("</Placemark>", 2);		
		return bff.toString();
	}
	
	private String getTrackDescription()
	{
		CustomStringBuffer bff = new CustomStringBuffer(1500);
		bff.appendTabbedLine("<description>", 2);
		bff.appendTabbedLine("<![CDATA[", 3);
		bff.appendTabbedLine("<table>", 4);		
		if(tracksInvalids)
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("Total number of locations: " + numberOfLocations), 5);
		bff.appendTabbedLine(HTMLUtils.htmlTableRow("Number of" + (tracksInvalids ? " valid" : "") + " locations: " + numberOfValidLocations), 5);
		if(tracksInvalids)
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("Number of invalid locations: " + (numberOfLocations - numberOfValidLocations)), 5);
		bff.appendTabbedLine(HTMLUtils.htmlTableRow("Starting time:" + XMLUtils.timeDateValue(trackStartTime)), 5);
		//Start&Stop time(s) & timespan(s):
		if(tracksInvalids && firstLocation != null)
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("Time of first (possibly invalid) location: " + XMLUtils.timeDateValue(firstLocationReceivalTime)), 5);
		if(firstValidLocation != null)
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("Time of first" + (tracksInvalids ? " valid" : "") + " location: " + XMLUtils.timeDateValue(firstValidLocation.getTimestamp())), 5);
		if(lastValidLocation != null)
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("Time of last"  + (tracksInvalids ? " valid" : "") + " location: " + XMLUtils.timeDateValue(lastValidLocation.getTimestamp())), 5);
		if(tracksInvalids && lastLocation != null)
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("Time of last (possibly invalid) location: " + XMLUtils.timeDateValue(lastLocationReceivalTime)), 5);
		if(tracksInvalids && firstLocation != null && lastLocation != null)
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("Time between first and last (possibly invalid) location: " + StringUtils.formatTimeSpan(lastLocationReceivalTime - firstLocationReceivalTime)), 5);
		if(firstValidLocation != null && lastValidLocation != null)
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("Time between first and last" + (tracksInvalids ? " valid" : "") + " location: " + StringUtils.formatTimeSpan(lastValidLocation.getTimestamp() - firstValidLocation.getTimestamp())), 5);
		bff.appendTabbedLine(HTMLUtils.htmlTableRow("Covered distance: " + coveredDistance + "m"), 5);
		if(minAltitude != Float.MAX_VALUE)
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("Minimum altitude: " + minAltitude + "m"), 5);
		if(maxAltitude != Float.MIN_VALUE)
			bff.appendTabbedLine(HTMLUtils.htmlTableRow("Maximum altitude: " + maxAltitude + "m"), 5);
		bff.appendTabbedLine(HTMLUtils.htmlTableRow("Location API settings: " + LocationAPIUtilities.getLocationAPISettings(locationProxy)), 5);
		bff.appendTabbedLine("</table>", 4);
		bff.appendTabbedLine("]]>", 3);
		bff.appendTabbedLine("</description>", 2);
		return bff.toString();
	}
	
	protected String coordinatesTuple(Coordinates coords)
	{
		return LocationAPIUtilities.coordinatesTuple(coords, false, ",");
	}

	private String kmlTimeSpan(long beginTime, long endTime, int tabs)
	{
		CustomStringBuffer bff = new CustomStringBuffer();
		bff.appendTabbedLine("<TimeSpan>", tabs);
		bff.appendTabbedLine("<begin>" + XMLUtils.timeDateValue(beginTime) + "</begin>", tabs + 1);
		bff.appendTabbedLine("<end>" + XMLUtils.timeDateValue(endTime) + "</end>", tabs + 1);
		bff.appendTabbedLine("</TimeSpan>", tabs);
		return bff.toString();
	}

}
